package org.beigesoft.handler;

/*
 * Copyright (c) 2019 Beigesoft™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

import java.util.Map;

import org.beigesoft.log.ILogger;
import org.beigesoft.model.IRequestData;

/**
 * <p>Spam/swindler request base handler.
 * Any service that reveal/suspect spam can invokes this.</p>
 *
 * @author Yury Demidenko
 */
public class SpamHnd implements ISpamHnd {

  /**
   * <p>Logger security.</p>
   **/
  private ILogger secLog;

  /**
   * <p>Handle suspected spam request.</p>
   * @param pRqVs Request scoped variables
   * @param pRqDt Request Data
   * @param pDang danger, e.g. 1 - wrong parameters,
   * 100 invasion attempt into authorized session
   * @param pMsg message maybe null
   * @throws Exception - an exception
   */
  @Override
  public final void handle(final Map<String, Object> pRqVs,
    final IRequestData pRqDt, final int pDang,
      final String pMsg) throws Exception  {
    String msg = "Spam request from host/addr/port/user/danger: " + pRqDt
      .getRemoteHost() + "/" + pRqDt.getRemoteAddr() + "/" + pRqDt
        .getRemotePort() + "/" + pRqDt.getRemoteUser() + "/" + pDang + ". ";
    if (pMsg != null) {
      msg += pMsg;
    }
    if (pDang < 10) {
      this.secLog.warn(pRqVs, SpamHnd.class, msg);
    } else {
      this.secLog.error(pRqVs, SpamHnd.class, msg);
    }
  }

  //Simple getters and setters:
  /**
   * <p>Getter for secLog.</p>
   * @return ILogger
   **/
  public final ILogger getSecLog() {
    return this.secLog;
  }

  /**
   * <p>Setter for secLog.</p>
   * @param pSecLog reference
   **/
  public final void setSecLog(final ILogger pSecLog) {
    this.secLog = pSecLog;
  }
}
