package org.beigesoft.service;

/*
 * Copyright (c) 2016 Beigesoft™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

import java.util.Map;

import org.beigesoft.model.ColumnsValues;

/**
 * <p>Helper to create Insert Update statement
 * by Android way (see ISrvDatabase).
 * JDBC SrvDatabase use it.
 * </p>
 *
 * @author Yury Demidenko
 */
public class HlpInsertUpdate {

  /**
   * <p>Evaluate SQL insert statement.
   * </p>
   * @param pTable table name
   * @param pColumnsVals type-safe map column name - column value
   * @return row count affected
   * @throws Exception - an exception
   **/
  public final String evalSqlInsert(final String pTable,
    final ColumnsValues pColumnsVals) throws Exception  {
    StringBuffer result = new StringBuffer("insert into "
      + pTable + " (");
    StringBuffer values = new StringBuffer(" values (");
    boolean isFirstField = true;
    ColumnsValues cv = filterCv(pColumnsVals, true);
    for (Map.Entry<String, Integer> entry : cv.getIntegersMap().entrySet()) {
      if (isFirstField) {
        isFirstField = false;
      } else {
        result.append(", ");
        values.append(", ");
      }
      result.append(entry.getKey().toUpperCase());
      values.append(cv.evalSqlValue(entry.getKey()));
    }
    for (Map.Entry<String, Long> entry : cv.getLongsMap().entrySet()) {
      if (isFirstField) {
        isFirstField = false;
      } else {
        result.append(", ");
        values.append(", ");
      }
      result.append(entry.getKey().toUpperCase());
      values.append(cv.evalSqlValue(entry.getKey()));
    }
    for (Map.Entry<String, String> entry : cv.getStringsMap().entrySet()) {
      if (isFirstField) {
        isFirstField = false;
      } else {
        result.append(", ");
        values.append(", ");
      }
      result.append(entry.getKey().toUpperCase());
      values.append(cv.evalSqlValue(entry.getKey()));
    }
    for (Map.Entry<String, Float> entry : cv.getFloatsMap().entrySet()) {
      if (isFirstField) {
        isFirstField = false;
      } else {
        result.append(", ");
        values.append(", ");
      }
      result.append(entry.getKey().toUpperCase());
      values.append(cv.evalSqlValue(entry.getKey()));
    }
    for (Map.Entry<String, Double> entry : cv.getDoublesMap().entrySet()) {
      if (isFirstField) {
        isFirstField = false;
      } else {
        result.append(", ");
        values.append(", ");
      }
      result.append(entry.getKey().toUpperCase());
      values.append(cv.evalSqlValue(entry.getKey()));
    }
    result.append(")\n");
    result.append(values + ")");
    return result.toString();
  }

  /**
   * <p>Evaluate SQL update statement.
   * </p>
   * @param pTable table name
   * @param pColumnsVals type-safe map column name - column value
   * @param pWhere where conditions e.g. "itsId=2 AND itsVersion=2"
   * @return row count affected
   * @throws Exception - an exception
   **/
  public final String evalSqlUpdate(final String pTable,
    final ColumnsValues pColumnsVals,
      final String pWhere) throws Exception {
    StringBuffer result = new StringBuffer("update "
      + pTable + " set \n");
    boolean isFirstField = true;
    ColumnsValues cv = filterCv(pColumnsVals, false);
    for (String key : cv.getIntegersMap().keySet()) {
      if (isFirstField) {
        isFirstField = false;
      } else {
        result.append(", ");
      }
      result.append(key.toUpperCase() + "=" + cv.evalSqlValue(key));
    }
    for (String key : cv.getLongsMap().keySet()) {
      if (isFirstField) {
        isFirstField = false;
      } else {
        result.append(", ");
      }
      result.append(key.toUpperCase() + "=" + cv.evalSqlValue(key));
    }
    for (String key : cv.getFloatsMap().keySet()) {
      if (isFirstField) {
        isFirstField = false;
      } else {
        result.append(", ");
      }
      result.append(key.toUpperCase() + "=" + cv.evalSqlValue(key));
    }
    for (String key : cv.getDoublesMap().keySet()) {
      if (isFirstField) {
        isFirstField = false;
      } else {
        result.append(", ");
      }
      result.append(key.toUpperCase() + "=" + cv.evalSqlValue(key));
    }
    for (String key : cv.getStringsMap().keySet()) {
      if (isFirstField) {
        isFirstField = false;
      } else {
        result.append(", ");
      }
      result.append(key.toUpperCase() + "=" + cv.evalSqlValue(key));
    }
    if (pWhere != null) {
    result.append(" where " + pWhere);
    }
    result.append(";\n");
    return result.toString();
  }

  /**
   * <p>Filters ColumnsValues from unneeded columns.
   * Test in android.sqlite.</p>
   * @param pColumnsVals Columns Values
   * @param pIns if insert, update otherwise
   * @return filtered Column Values
   **/
  public final ColumnsValues filterCv(
    final ColumnsValues pColumnsVals, final boolean pIns) {
    ColumnsValues cv = new ColumnsValues();
    cv.setIdColumnsNames(pColumnsVals.getIdColumnsNames());
    cv.getFormula().addAll(pColumnsVals.getFormula());
    for (Map.Entry<String, Integer> entry
      : pColumnsVals.getIntegersMap().entrySet()) {
      boolean notIdCol = pColumnsVals.getIdColumnsNames() == null
        || !isArrContains(entry.getKey(), pColumnsVals.getIdColumnsNames());
      if (notIdCol || pIns && entry.getValue() != null) {
        cv.put(entry.getKey(), entry.getValue());
      }
    }
    for (Map.Entry<String, Long> entry
      : pColumnsVals.getLongsMap().entrySet()) {
      boolean notIdCol = pColumnsVals.getIdColumnsNames() == null
        || !isArrContains(entry.getKey(), pColumnsVals.getIdColumnsNames());
      boolean notOldVer = !ISrvOrm.VERSIONOLD_NAME.equals(entry.getKey());
      if (notOldVer && (notIdCol || pIns && entry.getValue() != null)) {
        cv.put(entry.getKey(), entry.getValue());
      }
    }
    for (Map.Entry<String, Float> entry
      : pColumnsVals.getFloatsMap().entrySet()) {
      cv.put(entry.getKey(), entry.getValue());
    }
    for (Map.Entry<String, Double> entry
      : pColumnsVals.getDoublesMap().entrySet()) {
      cv.put(entry.getKey(), entry.getValue());
    }
    for (Map.Entry<String, String> entry
      : pColumnsVals.getStringsMap().entrySet()) {
      cv.put(entry.getKey(), entry.getValue());
    }
    return cv;
  }

  /**
   * <p>Check if value is in array.</p>
   * @param <T> type value
   * @param pVal value
   * @param pArr array
   * @return if contains
   **/
  protected final <T> boolean isArrContains(final T pVal,
    final T[] pArr) {
    for (T val : pArr) {
      if (val.equals(pVal)) {
        return true;
      }
    }
    return false;
  }
}
